@@ -1,4 +1,6 @@
-from pycstruct.pycstruct import BaseDef
 from pycstruct.pycstruct import StructDef
 from pycstruct.pycstruct import BitfieldDef
-from pycstruct.pycstruct import EnumDef
\ No newline at end of file
+from pycstruct.pycstruct import EnumDef
+
+from pycstruct.cparser import parse_file
+from pycstruct.cparser import parse_str
\ No newline at end of file
